# command_resolve.py
#
# Copyright (C) 2014 Tim Marston <tim@edm.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys, re, getopt
import the
from deployment import Deployment


class CommandResolve:


	def __init__( self ):
		self.repo = None


	def print_help( self ):
		print "Usage: " + the.program.name + " resolve [--repo=REPO]"
		print
		#      01234567890123456789012345678901234567890123456789012345678901234567890123456789
		print "Reattempt to deploy files following an update that failed due to conflicts."
		print
		print "After an update, no files will have been redeployed from the local repository to"
		print "your home directory if there were merge or deployment conflicts.  In a situation"
		print "such as this, you are expected to manually intervene; fixing broken merged files"
		print "by hand and moving aside files in your home directory.  After having done this,"
		print "this command can be used to reattempt the deployment and finalise the update."
		print
		print "You can list outstanding conflicts by typing:"
		print "    " + the.program.name + " conflicts"
		print
		print "Options:"
		print "  -r, --repo=REPO  select the repo to check-out or create (defaults to 'home')"
		print "  -v, --verbose    display information about what is being done"
		print "      --help       display help and exit"
		exit( 0 )


	def parse_command_line( self ):
		opts, args = getopt.gnu_getopt(
			sys.argv[ 1: ], "r:v",
			[ "repo=", "verbose", "help" ] )
		for opt, optarg in opts:
			if opt in [ '--repo', '-r' ]:
				if not re.match( '^[-a-zA-z0-9.]+$', optarg ):
					raise the.program.FatalError(
						'invalid repository name: ' + optarg )
				self.repo = optarg
			elif opt in [ '--verbose', '-v' ]:
				the.verbose = True
			elif opt == "--help":
				self.print_help()
		
		# discard first argument (the command)
		args.pop( 0 )

		# remaining arguments
		if len( args ):
			raise the.program.UsageError( 'too many arguments' )


	def run( self ):

		# set up repo and check it exists
		the.set_repo( self.repo )
		the.repo.check_dir_exists()

		# initialise deployment (check it's valid)
		deployment = Deployment()
		deployment.check_ongoing( True )

		# check for conflicts
		message = ''
		conflicts = the.repo.vcs.has_conflicts()
		if conflicts:
			message += 'Conflicts in %s:\n  %s' % \
					   ( the.repo.name, '\n  '.join( conflicts ) )
		conflicts = deployment.check_conflicts()
		if conflicts:
			message += 'Deployment conflicts:\n  %s' % \
					   '\n  '.join( conflicts )
		if message:
			raise the.program.FatalError( 'there are conflicts:\n' + message )

		# copy-out changes from repo
		try:
			deployment.copy_out()
		except deployment.Conflict as e:
			raise the.program.FatalError( e.msg )

		# now we've copied-out, revert any copied-in changes!
		if the.verbose: print "reverting %s" % the.repo.dir
		the.repo.vcs.revert()
